#[macro_export]
macro_rules! handler {
    ($v: vis $function_name: ident ($($param: ident: $type: ty),*), $message: literal, $next_state: expr, $next_func: ident) => {
        #[inline]
        $v async fn $function_name(
            bot: Throttle<Bot>,
            msg: Message,
            _: DatabaseConnection,
            dialogue: MainDialogue,
            $($param: $type),*
        ) -> $crate::Result<()> {
            let previous = bot.send_message(msg.chat.id, $message).await?;
            dialogue
                .update($next_state(Handler::new(
                    move |bot, msg, db, dialogue, param| Box::pin($next_func(bot, msg, db, dialogue, $($param,)* param)),
                    &previous,
                )))
                .await?;
            Ok(())
        }
    };
}

#[macro_export]
macro_rules! ask_name_handler {
    ($v: vis $function_name: ident ($($param: ident: $type: ty),*), $message: literal, $next_func: ident) => {
        #[inline]
        $v async fn $function_name(
            bot: Throttle<Bot>,
            msg: Message,
            dialogue: MainDialogue,
            db: DatabaseConnection,
            $($param: $type),*
        ) -> $crate::Result<()> {
            let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
            let markup = account_markup(user_id, &db).await?;
            if markup.keyboard.is_empty() {
                bot.send_message(msg.chat.id, "No accounts found")
                    .reply_markup(deletion_markup())
                    .await?;
                return Ok(());
            }
            let previous = bot
                .send_message(msg.chat.id, $message)
                .reply_markup(markup)
                .await?;
            dialogue
                .update(State::GetExistingName(Handler::new(
                    move |bot, msg, db, dialogue, param| Box::pin($next_func(bot, msg, db, dialogue, $($param,)* param)),
                    &previous,
                )))
                .await?;
            Ok(())
        }
    };
}

#[macro_export]
macro_rules! simple_state_handler {
    ($function_name: ident, $check: ident, $no_text_message: literal) => {
        #[inline]
        pub async fn $function_name(
            bot: Throttle<Bot>,
            msg: Message,
            db: DatabaseConnection,
            dialogue: MainDialogue,
            next: PackagedHandler<String>,
        ) -> $crate::Result<()> {
            super::generic::generic(
                bot,
                msg,
                db,
                dialogue,
                |bot, msg, db, param| Box::pin($check(bot, msg, db, param)),
                $no_text_message,
                next,
            )
            .await
        }
    };
}
